@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require 'faraday'
 require 'forwardable'
 require 'digest/sha1'
@@ -6,6 +8,7 @@ module FaradayMiddleware
   # Public: Caches GET responses and pulls subsequent ones from the cache.
   class Caching < Faraday::Middleware
     attr_reader :cache
+
     # Internal: List of status codes that can be cached:
     # * 200 - 'OK'
     # * 203 - 'Non-Authoritative Information'
@@ -14,7 +17,7 @@ module FaradayMiddleware
     # * 302 - 'Found'
     # * 404 - 'Not Found'
     # * 410 - 'Gone'
-    CACHEABLE_STATUS_CODES = [200, 203, 300, 301, 302, 404, 410]
+    CACHEABLE_STATUS_CODES = [200, 203, 300, 301, 302, 404, 410].freeze
 
     extend Forwardable
     def_delegators :'Faraday::Utils', :parse_query, :build_query
@@ -23,30 +26,36 @@ module FaradayMiddleware
     #
     # cache   - An object that responds to read and write (default: nil).
     # options - An options Hash (default: {}):
-    #           :ignore_params - String name or Array names of query params
-    #                            that should be ignored when forming the cache
-    #                            key (default: []).
-    #           :write_options - Hash of settings that should be passed as the third
-    #                            options parameter to the cache's #write method. If not
-    #                            specified, no options parameter will be passed.
+    #           :ignore_params - String name or Array names of query
+    #                            params that should be ignored when forming
+    #                            the cache key (default: []).
+    #           :write_options - Hash of settings that should be passed as the
+    #                            third options parameter to the cache's #write
+    #                            method. If not specified, no options parameter
+    #                            will be passed.
+    #           :full_key      - Boolean - use full URL as cache key:
+    #                            (url.host + url.request_uri)
     #
     # Yields if no cache is given. The block should return a cache object.
     def initialize(app, cache = nil, options = {})
       super(app)
-      options, cache = cache, nil if cache.is_a? Hash and block_given?
+      if cache.is_a?(Hash) && block_given?
+        options = cache
+        cache = nil
+      end
       @cache = cache || yield
       @options = options
     end
 
     def call(env)
-      if :get == env[:method]
+      if env[:method] == :get
         if env[:parallel_manager]
           # callback mode
           cache_on_complete(env)
         else
           # synchronous mode
           key = cache_key(env)
-          unless response = cache.read(key) and response
+          unless (response = cache.read(key)) && response
             response = @app.call(env)
             store_response_in_cache(key, response)
           end
@@ -61,24 +70,29 @@ module FaradayMiddleware
       url = env[:url].dup
       if url.query && params_to_ignore.any?
         params = parse_query url.query
-        params.reject! {|k,| params_to_ignore.include? k }
+        params.reject! { |k,| params_to_ignore.include? k }
         url.query = params.any? ? build_query(params) : nil
       end
       url.normalize!
-
-      Digest::SHA1.hexdigest(url.request_uri)
+      digest = full_key? ? url.host + url.request_uri : url.request_uri
+      Digest::SHA1.hexdigest(digest)
     end
 
     def params_to_ignore
-      @params_to_ignore ||= Array(@options[:ignore_params]).map { |p| p.to_s }
+      @params_to_ignore ||= Array(@options[:ignore_params]).map(&:to_s)
+    end
+
+    def full_key?
+      @full_key ||= @options[:full_key]
     end
 
     def cache_on_complete(env)
       key = cache_key(env)
-      if cached_response = cache.read(key)
+      if (cached_response = cache.read(key))
         finalize_response(cached_response, env)
       else
-        # response.status is nil at this point, any checks need to be done inside on_complete block
+        # response.status is nil at this point
+        # any checks need to be done inside on_complete block
         @app.call(env).on_complete do |response_env|
           store_response_in_cache(key, response_env.response)
           response_env
